///*|-----------------------------------------------------------------------------
// *|            This source code is provided under the Apache 2.0 license      --
// *|  and is provided AS IS with no warranty or guarantee of fit for purpose.  --
// *|                See the project's LICENSE.md for details.                  --
// *|             Copyright Refinitiv 2021. All rights reserved.                --
///*|-----------------------------------------------------------------------------

#include "Step3.h"

using namespace refinitiv::ema::access;
using namespace refinitiv::ema::rdm;
using namespace std;


void AppClient::onRefreshMsg( const RefreshMsg& refreshMsg, const OmmConsumerEvent& event) {
	cout << endl << "----------------------------------" << endl;
	// an item handle uniquely identifies the request and is used to unsubscribe it
	// a closure is any user owned object passed on when making a request and is passed back in response (refresh/update/status) messages
	cout << "Refresh message, item Handle: " << event.getHandle() << " Closure: " << event.getClosure() << endl;
	
	//  display item and service name
	if( refreshMsg.hasMsgKey() )
		cout << endl << "Item Name: " << refreshMsg.getName() << endl << "Service Name: " << refreshMsg.getServiceName();

	//  display the status of current request
	cout << endl << "Item State: " << refreshMsg.getState().toString() << endl;

	// Level1 data payload is encoded as an OMM FieldList or NoData if no payload data is available
	if( DataType::FieldListEnum == refreshMsg.getPayload().getDataType() )
		decode( refreshMsg.getPayload().getFieldList() );

	cout << endl << "Waiting for updates. Application will exit in 30 seconds..." << endl;
}


void AppClient::onUpdateMsg( const UpdateMsg& updateMsg, const OmmConsumerEvent& ) {
	//  display item and service name
	if( updateMsg.hasMsgKey() )
		cout << endl << "Item Name: " << updateMsg.getName() << endl << "Service Name: " << updateMsg.getServiceName() << endl;

	// Level2 data payload is encoded as an OMM FieldList or NoData if no payload data is available
	if (DataType::FieldListEnum == updateMsg.getPayload().getDataType())
		decode( updateMsg.getPayload().getFieldList() );
}



void AppClient::onStatusMsg( const StatusMsg& statusMsg, const OmmConsumerEvent& )	{
	cout << "Status message" << endl;

	//  display item and service name
	if( statusMsg.hasMsgKey() )
		cout << endl << "Item Name: " << statusMsg.getName() << ", Service Name: " << statusMsg.getServiceName() << endl;

	if( statusMsg.hasState() )	{
		//  display the status of current request
		cout << "Stream state: " << statusMsg.getState().getStreamState()
			<< "\tStatus code: " << statusMsg.getState().getStatusCode()
			<< "\tStatus text: " << statusMsg.getState().getStatusText() << endl;
	}
}


void AppClient::decode( const FieldList& fl )	{
	// recurse over all field entries in list
	while( fl.forth() )	{
		const FieldEntry& fe = fl.getEntry();
	
		cout << "Fid: " << fe.getFieldId() << " Name: " << fe.getName() << " DataType: " << DataType( fe.getLoadType() ).toString() << " Value: ";

		// decode primitive OMM data type of this entry
		if( fe.getCode() == Data::BlankEnum )
			cout << " blank" << endl;
		else	{
			switch ( fe.getLoadType() )		{
				case DataType::RealEnum :
					cout << fe.getReal().getAsDouble() << endl;
					break;
				case DataType::DateEnum :
					cout << (UInt64)fe.getDate().getDay() << " / " << (UInt64)fe.getDate().getMonth() << " / " << (UInt64)fe.getDate().getYear() << endl;
					break;
				case DataType::TimeEnum :
					cout << (UInt64)fe.getTime().getHour() << ":" << (UInt64)fe.getTime().getMinute() << ":" << (UInt64)fe.getTime().getSecond() << ":" << (UInt64)fe.getTime().getMillisecond() << endl;
					break;
				case DataType::IntEnum :
					cout << fe.getInt() << endl;
					break;
				case DataType::UIntEnum :
					cout << fe.getUInt() << endl;
					break;
				case DataType::AsciiEnum :
					cout << fe.getAscii() << endl;
					break;
				case DataType::EnumEnum :
					cout << fe.getEnum() << endl;
					break;
				case DataType::ErrorEnum :
					cout << "( " << fe.getError().getErrorCodeAsString() << " )" << endl;
				default:
					cout << endl;
					break;
			}
		}
	}
}



void AppClient::decode( const Map& map )	{
}	



int main( int argc, char* argv[] )	{
	
	try {
		// instantiate callback client
		AppClient client;
		
		cout << "Connecting to market data server" << endl;

		// create OMM consumer
		OmmConsumer consumer( 
			OmmConsumerConfig()
				.host( "ADS:14002" )
				.username( "user" ) 
		);
		
		cout << "Subscribing to market data" << endl;

		// subscribe to Level1 market data
		consumer.registerClient( 
			ReqMsg()
				// default subscription domain is MMT_MARKET_PRICE
				.serviceName( "ELEKTRON_DD" )
				.name( "IBM.N" ),
			client );
		
		// block this thread for a while
		sleep(30000);
		
	} 
	catch ( const OmmException& excp ) {
		cout << "Exception subscribing to market data: " << excp << endl;
	}
	
	return 0;
}
